@@ -3,6 +3,7 @@
 #include <stdlib.h>
 
 #include "pyclutter.h"
+#include "pyclutter-private.h"
 
 struct _PyClutterCallback
 {
@@ -172,6 +173,15 @@ pyclutter_color_from_pyobject (PyObject     *object,
                 return TRUE;
         }
 
+        if (PyString_Check (object)) {
+                const char *str = PyString_AsString (object);
+
+                if (!clutter_color_from_string (color, str))
+                        goto out;
+
+                return TRUE;
+        }
+
         if (PyTuple_Check (object) && (PyTuple_Size (object) == 4)) {
                 int i;
 
@@ -202,6 +212,34 @@ out:
         return FALSE;
 }
 
+gboolean
+pycogl_color_from_pyobject (PyObject *object, CoglColor *color)
+{
+  g_return_val_if_fail (color != NULL, FALSE);
+  float red, green, blue, alpha = 1.0;
+
+  /* XXX: I might remove this in future */
+  if (pyg_boxed_check (object, CLUTTER_TYPE_COLOR))
+    {
+      ClutterColor *c = pyg_boxed_get (object, ClutterColor);
+      cogl_color_set_from_4ub (color, c->red, c->green, c->blue, c->alpha);
+      return TRUE;
+    }
+
+  if (!PyTuple_Check (object) || !PyArg_ParseTuple (object, "fff|f",
+                                                    &red, &green, &blue,
+                                                    &alpha))
+    {
+      PyErr_Clear ();
+      PyErr_SetString (PyExc_ValueError,
+                       "color must be a 3 or 4-tuple of float from 0.0-1.0");
+      return FALSE;
+    }
+
+  cogl_color_set_from_4f (color, red, green, blue, alpha);
+  return TRUE;
+}
+
 gboolean
 pyclutter_actor_box_from_pyobject (PyObject        *object,
                                    ClutterActorBox *box)
@@ -219,14 +257,14 @@ pyclutter_actor_box_from_pyobject (PyObject        *object,
                 for (i = 0; i < 4; i++) {
                         PyObject *comp = PyTuple_GetItem (object, i);
 
-                        if (!PyInt_Check (comp))
+                        if (!PyFloat_Check (comp))
                                 goto out;
 
                         switch (i) {
-                        case 0: box->x1 = PyInt_AsLong (comp); break;
-                        case 1: box->y1 = PyInt_AsLong (comp); break;
-                        case 2: box->x2 = PyInt_AsLong (comp); break;
-                        case 3: box->y2 = PyInt_AsLong (comp); break;
+                        case 0: box->x1 = PyFloat_AsDouble (comp); break;
+                        case 1: box->y1 = PyFloat_AsDouble (comp); break;
+                        case 2: box->x2 = PyFloat_AsDouble (comp); break;
+                        case 3: box->y2 = PyFloat_AsDouble (comp); break;
                         default:
                                 g_assert_not_reached ();
                                 break;
@@ -243,14 +281,14 @@ out:
         return FALSE;
 }
 
-guint32
+gdouble
 pyclutter_alpha_func (ClutterAlpha *alpha,
                       gpointer      data)
 {
         PyClutterCallback *pycb = data;
         PyGILState_STATE state;
         PyObject *py_alpha, *retobj;
-        guint32 retval = 0;
+        gdouble retval = 0.0;
 
         state = pyg_gil_state_ensure ();
 
@@ -259,13 +297,9 @@ pyclutter_alpha_func (ClutterAlpha *alpha,
         if (retobj == NULL)
                 PyErr_Print ();
 
-        if (PyLong_Check (retobj)) {
-                retval = (guint32) PyLong_AsUnsignedLong (retobj);
-        }
-        else if (PyInt_Check (retobj)) {
-                retval = (guint32) PyInt_AsLong (retobj);
-        }
-        else {
+        if (PyFloat_Check (retobj)) {
+                retval = (gdouble) PyFloat_AsDouble (retobj);
+        } else {
                 PyErr_SetString (PyExc_TypeError,
                                  "returned value is not an integer");
         }
@@ -278,25 +312,37 @@ pyclutter_alpha_func (ClutterAlpha *alpha,
         return retval;
 }
 
-void
-pyclutter_effect_complete (ClutterActor *actor,
-                           gpointer      data)
+const char *
+pyclutter_event_get_name (const ClutterEvent *event)
 {
-        PyClutterCallback *pycb = data;
-        PyGILState_STATE state;
-        PyObject *py_actor, *retobj;
+        if (event == NULL)
+                return "ClutterEvent";
 
-        state = pyg_gil_state_ensure ();
+        switch (event->type) {
+                case CLUTTER_BUTTON_PRESS:
+                case CLUTTER_BUTTON_RELEASE:
+                        return "ClutterButtonEvent";
 
-        py_actor = pygobject_new ((GObject *) actor);
-        retobj = pyclutter_callback_invoke (pycb, py_actor);
-        if (retobj == NULL)
-                PyErr_Print ();
+                case CLUTTER_KEY_PRESS:
+                case CLUTTER_KEY_RELEASE:
+                        return "ClutterKeyEvent";
 
-        Py_XDECREF (retobj);
-	Py_XDECREF (py_actor);
+                case CLUTTER_ENTER:
+                case CLUTTER_LEAVE:
+                        return "ClutterCrossingEvent";
 
-        pyg_gil_state_release (state);
+                case CLUTTER_MOTION:
+                        return "ClutterMotionEvent";
 
-        pyclutter_callback_free (pycb);
+                case CLUTTER_STAGE_STATE:
+                        return "ClutterStageStateEvent";
+
+                case CLUTTER_SCROLL:
+                        return "ClutterScrollEvent";
+
+                default:
+                        return "ClutterEvent";
+        }
+
+        return NULL;
 }
